#!/usr/bin/env python3

import argparse


def get_id(x, idmap):
    if x in idmap:
        return str(idmap[x])
    elif x[0] == '-' and x[1:] in idmap:
        return '-{}'.format(idmap[x[1:]])
    return x

def main():

    ap = argparse.ArgumentParser()
    ap.add_argument('btor1_input')
    ap.add_argument('btor2_output')
    args = ap.parse_args()

    with open(args.btor1_input, 'r') as infile:

        lines = []
        idmap = dict()
        bwsortmap = dict()
        states = dict()
        vars = dict()
        change_bw = dict()
        for line in infile:
            lines.append(line)

            fields = line.split()
            if len(fields) <= 2 or line.startswith(';'):
                continue

            id = fields[0]
            op = fields[1]
            bw = fields[2]

            assert op not in ['array', 'write', 'read', 'anext', 'acond']
            assert id not in idmap

            idmap[id] = 0

            if bw not in bwsortmap:
                bwsortmap[bw] = 0

            if op == 'next':
                states[fields[3]] = True
            elif op == 'var':
                vars[id] = True

            if op in ['sll', 'srl', 'sra']:
                change_bw[fields[4]] = bw


        new_lines = []
        while lines[0].startswith(';'):
            new_lines.append(lines.pop(0).strip())

        # Create sorts
        bws = list(sorted(bwsortmap.keys(), key=lambda x: int(x)))
        id = 1
        for bw in bws:
            new_lines.append('{} sort bitvec {}'.format(id, bw))
            bwsortmap[bw] = id
            id += 1

        zeroes = {}
        for line in lines:
            if line.startswith(';') or line == '\n':
                new_lines.append(line)
                continue

            fields = line.strip().split()
            assert len(fields) > 3

            old_id = fields[0]
            op = fields[1]
            bw = fields[2]

            if op == 'var':
                if old_id in states:
                    op = 'state'
                else:
                    op = 'input'
            elif op == 'root':
                op = 'bad'
            elif op == 'cond':
                op = 'ite'

            # Make sure to create the init-value before the state
            if op == 'state' and bw not in zeroes:
                new_lines.append('{} zero {}'.format(id, bwsortmap[bw]))
                zeroes[bw] = id
                id += 1

            idmap[old_id] = id

            new_line = []
            new_line.append(str(idmap[old_id]))
            new_line.append(op)
            if op != 'bad':
                if old_id in change_bw:
                    if op == 'slice':
                        # In some cases fields[3] had to be sliced in order
                        # to be used in a shift. This is not necessary anymore.
                        idmap[old_id] = idmap[fields[3]]
                        continue
                    bw = change_bw[old_id]
                new_line.append(str(bwsortmap[bw]))

            if op == 'slice':
                new_line.append(get_id(fields[3], idmap))
                new_line.extend(fields[4:])
            elif op in ['const', 'constd', 'consth']:
                new_line.extend(fields[3:])
            else:
                new_line.extend([get_id(x, idmap) for x in fields[3:]])

            new_lines.append(' '.join(new_line))
            id += 1

            if op == 'state':
                state_id = id - 1
                zero_id = zeroes[bw]
                sort_id = bwsortmap[bw]
                # 0-initialize states
                new_lines.append('{} init {} {} {}'.format(id, sort_id, state_id, zero_id))
                id += 1


    with open(args.btor2_output, 'w') as outfile:
        outfile.write('\n'.join(new_lines))


if __name__ == '__main__':
    main()
